// @flow
/**
 * Copyright (c) 2017-present, Evosphere.
 * All rights reserved.
 */

import Container from '../util/Container';

export type ServiceType = Object;
export type ParameterType = mixed;
export type ServiceContainerType = {[key: string] : ServiceType};

/**
 * Allows the Kernel o have a ServiceContainer
 * @param Kernel
 */
export default function containerMixin(Kernel : any){
  Kernel.prototype._services = Container.create();
  Kernel.prototype._parameters = Container.create();

  /**
   * Sets a service
   * @param name
   * @param service
   */
  Kernel.prototype.set = function(name : string, service : ServiceType){
    if(typeof service === "function"){
      this._services.set(name, service(this._services));
    } else {
      this._services.set(name, service);
    }

    return this;
  };

  /**
   * Sets the parameter
   * @param name
   * @param parameter
   */
  Kernel.prototype.setParameter = function(name : string, parameter : mixed) {
    this._parameters.set(name, parameter);
    return this;
  };

  /**
   * Returns a service
   * If the name starts with a "#", it returns a parameter instead
   * @param name
   * @returns {*}
   */
  Kernel.prototype.get = function(name : string) : ServiceType | ParameterType {
    if(name.startsWith("#")) {
      return this._parameters.get(name.substr(1));
    }

    return this._services.get(name);
  };

  /**
   * Return all the services
   * @returns {BagType.<string, *>}
   */
  Kernel.prototype.getServices = function() : ServiceContainerType {
    return this._services.all();
  };

  /**
   * Loads a map of services where keys are the names
   * @param services
   */
  Kernel.prototype.loadServices = function(services : ?ServiceContainerType){
    if(services === null || services === undefined){
      return;
    }

    if(typeof services !== "object"){
      throw new Error("Services expect an object");
    }

    Object.keys(services).forEach(function(key){
      // $FlowFixMe
      this.set(key, services[key]);
    }.bind(this))
  };

  /**
   * Loads a map of parameters
   * @param parameters
   */
  Kernel.prototype.loadParameters = function(parameters : {[key: string] : mixed }) {
    if(parameters === null || parameters === undefined){
      return;
    }

    if(typeof parameters !== "object"){
      throw new Error("Parameters expect an object");
    }

    Object.keys(parameters).forEach(function(key){
      this.setParameter(key, parameters[key]);
    }.bind(this))
  }
}