/**
 * Copyright (c) 2017-present, Evosphere.
 * All rights reserved.
 */

import type { TaskType, KernelFactoryType } from "./definitionTypes"

const emptyArray = [];

/**
 * Throws an exception if the value isn't an array
 * @param value
 */
function assertIsArray(value : mixed) : void {
  if(Array.isArray(value) === false){
    // $FlowFixMe
    throw new Exception(
      "calling tasks require the second argument to be an array of args to pass to the function, " + typeof value + " given"
    );
  }
}

/**
 * Tasks are a way to influence the setup mechanism of the kernel.
 * It allows third party libraries to add steps and functionnalities to it, such as
 * customizing a service with custom configuration step.
 *
 * @param KernelFactory
 */
export default function(KernelFactory : KernelFactoryType) {
  KernelFactory.prototype.tasks = emptyArray;

  /**
   * Return the list of tasks
   * @returns {*}
   */
  KernelFactory.prototype.getTasks = function() : Array<TaskType> {
    return this.tasks;
  };

  /**
   * Clear the task list
   */
  KernelFactory.prototype.clearTaskRegistry = function() : void {
    this.tasks = emptyArray;
  };

  /**
   * Register the tasks
   * @param tasks
   */
  KernelFactory.prototype.registerTasks = function(tasks : Array<TaskType>) : void {
    this.tasks = tasks;
  };

  /**
   * Execute the tasks
   * @param name
   * @param args
   * @returns {Promise.<void>}
   */
  KernelFactory.prototype.executeTasks = async function(name : string, args : Array<TaskType>) : Promise<void> {
    assertIsArray(args);
    for(let i = 0; i < this.tasks.length; i++){
      if (typeof this.tasks[i][name] === "function") {
        await this.tasks[i][name].apply(this.tasks[i], args);
      }
    }
  };

  /**
   * Execute all the tasks matching the name and return their values
   *
   * @param name
   * @param args
   * @returns {Array}
   */
  KernelFactory.prototype.executeTasksAndReturnValue = function(name : string, args : Array<TaskType>) : Array<mixed> {
    assertIsArray(args);
    return this.tasks.filter(task => typeof task[name] === "function").map(task => task[name].apply(task, args));
  }
}